@@ -10,10 +10,10 @@ if (Sys.getenv("_LOCAL_TESTS_")=="TRUE") { ## set in etc/Renviron.site (cf R Win
       while (dev.cur()>1L) dev.off()
       op <- devAskNewPage(ask=FALSE)
       testfiles <- dir(paste0(projpath(),"/package/tests/testthat/"),pattern="*.R",full.names = TRUE)
-      # oldmaxt <- spaMM.options(example_maxtime=70)
+      # oldmaxt <- spaMM.options(example_maxtime=60)
       timings <- t(sapply(testfiles, function(fich){system.time(source(fich))}))
       # spaMM.options(oldmaxt)
-      print(colSums(timings))
+      print(sums <- colSums(timings))
       ## testthat::test_package(pkg) ## for an installed package
       if (FALSE) { ## tests not included in package (using unpublished data, etc.)
         priv_testfiles <- dir(paste0(projpath(),"/package/tests_private/"),pattern="*.R",full.names = TRUE)
@@ -26,10 +26,13 @@ if (Sys.getenv("_LOCAL_TESTS_")=="TRUE") { ## set in etc/Renviron.site (cf R Win
         print(colSums(priv_timings))
       }
       devAskNewPage(op)
+      if (FALSE) {
+        save(timings,file=paste0(projpath(),"/timings_",packageVersion("spaMM"),"_",sums[[1]],"s.rda"))
+      }
     } else if (FALSE) { ## for R CMD check (but still assuming _LOCAL_TESTS_), but this does not work on nested files
       library("testthat") # cf ?test_check for using library() here:
       library(pkg, character.only = TRUE)
-      oldmaxt <- spaMM.options(example_maxtime=70) ## then slow (Rstudio -> devtools tests) 
+      oldmaxt <- spaMM.options(example_maxtime=60) ## then slow (Rstudio -> devtools tests) 
       report <- test_check(pkg) 
       spaMM.options(oldmaxt)
       print(warnings()) # TODO? catch most of these by expect_warning(..)
