@@ -19,7 +19,11 @@ data("scotlip")
 # CAR by Laplace with 'inner' estimation of rho
 blob1 <- HLCor(cbind(npos,nneg)~1 +adjacency(1|gridcode),
           adjMatrix=Nmatrix,family=binomial(probit),data=donn,HLmethod="ML",control.HLfit = list(LevenbergM=FALSE)) ## 2 s.
-try(testthat::expect_true(abs(get_ranPars(blob1,which = "corrPars")[["1"]]$rho- 0.07961275)<1e-9))
+crit <- diff(range(get_ranPars(blob1,which = "corrPars")[["1"]]$rho, 0.07961275))
+if (spaMM.getOption("fpot_tol")>0) {
+  testthat::test_that(paste0("criterion was ",signif(crit,6)," from 0.07961275"), testthat::expect_true(crit<5e-9)) # bobyqa finds 0.04582924 ('flat' p_bv)
+} else testthat::expect_true(crit<5e-9)
+
 #AIC(blob1)
 VarCorr(blob1)
 
